import model.AdminUser
import model.DefaultUser
import model.ManagerUser
import model.User
import service.AdminServiceImpl
import service.AuthorizeService
import service.PasswordService
import java.time.LocalDate

val authorizationList = mutableListOf<User>()

fun main() {
    val passwordService = PasswordService()
    val adminService = AdminServiceImpl(authorizationList, passwordService)
    val authorizeService = AuthorizeService(authorizationList, passwordService)

    while (true) {
        println("1) Регистрация")
        println("2) Авторизация")
        val choice = readln()
        when (choice) {
            "1" -> {
                val user = createUser(passwordService)
                val registered = authorizeService.registration(user)
                if (registered) {
                    println("Регистрация успешна")
                } else {
                    println("Пользователь с таким email или логином уже существует")
                }
            }
            "2" -> {
                try {
                    println("Введите email:")
                    val email = readln()
                    println("Введите пароль:")
                    val password = readln()
                    val user = authorizeService.authorize(email, password)
                    println("Пользователь найден: ${user.login}, тип: ${user::class.simpleName}")

                    if (user is AdminUser) {
                        var adminFlg = true
                        while (adminFlg) {
                            adminFlg = adminFunctional(adminService)
                        }
                    }
                } catch (e: IllegalArgumentException) {
                    println(e.message)
                }
            }
            else -> println("Неверный выбор")
        }
    }
}

fun createUser(passwordService: PasswordService): User {
    println("Введите email:")
    val email = readln()
    require(email.contains("@")) { "Email должен содержать '@'" }

    println("Введите пароль:")
    val password = readln()
    require(password.length >= 8 && password.any { it.isDigit() } && password.any { it.isLetter() }) {
        "Пароль должен быть не менее 8 символов и содержать буквы и цифры"
    }

    println("Повторите пароль:")
    val confirmPassword = readln()
    require(password == confirmPassword) { "Пароли не совпадают" }

    println("Введите логин:")
    val login = readln()
    require(login.length >= 4) { "Логин должен быть не менее 4 символов" }

    println("Выберите роль: 1) Обычный пользователь 2) Администратор 3) Менеджер")
    val role = readln()

    val hashedPassword = passwordService.hashPassword(password)

    return when (role) {
        "1" -> DefaultUser(
            userId = authorizationList.size + 1,
            login = login,
            password = hashedPassword,
            email = email,
            lastAuthorizeDate = LocalDate.now()
        )
        "2" -> AdminUser(
            userId = authorizationList.size + 1,
            login = login,
            password = hashedPassword,
            email = email,
            lastAuthorizeDate = LocalDate.now()
        )
        "3" -> ManagerUser(
            userId = authorizationList.size + 1,
            login = login,
            password = hashedPassword,
            email = email,
            lastAuthorizeDate = LocalDate.now()
        )
        else -> throw IllegalArgumentException("Некорректный выбор роли")
    }
}

fun adminFunctional(adminService: AdminServiceImpl): Boolean {
    println("1) Показать всех")
    println("2) Показать всех сорт")
    println("3) Поменять пароль")
    println("4) Удалить чела")
    println("5) Пока")

    var flg = readln()
    when(flg) {
        "1" -> {
            adminService.getAllUsers().forEach {
                println("${it.userId}: ${it.login} (${it.email})")
            }
        }
        "2" -> {
            adminService.getAllUsersSort().forEach {
                println("${it.userId}: ${it.login} (${it.lastAuthorizeDate})")
            }
        }
        "3" -> {
            println("Введите email пользователя для обновления пароля")
            val email = readln()
            println("Введите новый пароль")
            val newPassword = readln()
            val updated = adminService.updateUserPassword(email, newPassword)
            if (updated) println("Пароль обновлен") else println("Пользователь не найден")
        }
        "4" -> {
            println("Введите ID пользователя для удаления")
            val userId = readln().toInt()
            val removed = adminService.removeUserById(userId)
            if (removed) println("Пользователь удален") else println("Пользователь не найден")
        }
        "5" -> return false
    }

    return true
}