package service

import model.User
import java.time.LocalDate

class AuthorizeService(
    private val authorizationList: MutableList<User>,
    private val passwordService: IPasswordService
) : IAuthorizeService {

    override fun registration(user: User): Boolean {
        if (authorizationList.any { it.email == user.email || it.login == user.login }) {
            return false // Пользователь с таким email или логином уже существует
        }
        authorizationList.add(user)
        return true
    }

    override fun authorize(email: String, password: String): User {
        val user = authorizationList.find { it.email == email }
            ?: throw IllegalArgumentException("Пользователь с таким email не найден")

        if (!passwordService.matches(password, user.password)) {
            throw IllegalArgumentException("Неверный пароль")
        }

        user.lastAuthorizeDate = LocalDate.now()
        return user
    }
}